#include <algorithm> // 'std::transform'
#include <string> // 'std::string'
#include <utility> // 'std::pair'

#include "Palindrome.h"
#include "Utilities.h"

std::string Substituable_character::toString() const {
	std::string result(1, character);
    // Lambda expression ('&' allows the injection of local variables, e.g., 'result', by reference):
    std::for_each(_substitutes.begin(), _substitutes.end(), [&](char substitute) /* -> void */ {
        result += "-" + std::string(1, substitute);
    });
    return result;
}

void Palindrome_word::alter(const Substituable_character& sc) {
    std::transform(_word.begin(), _word.end(), _word.begin(), sc); // 'sc' object is viewed as functor...
}

void Palindrome_word::alter_(const Substituable_character& sc) {
    auto my_lambda = [sc](char substitute) /* -> const char */ { // Format: https://docs.microsoft.com/fr-fr/cpp/cpp/media/lambdaexpsyntax.png?view=vs-2019
        // By lazyness, one calls 'const char operator() (char substitute) const' in 'Substituable_character':
        return sc(substitute);
    };
    std::transform(_word.begin(), _word.end(), _word.begin(), my_lambda); // Lambda expression instead of explicit functor...
}

// Initialization operates through "aggregate type":
const std::array<std::pair<char, char>, 2> _Intervals = {std::pair<char, char>('a', 'z'), std::pair<char, char>('A', 'Z')};

Palindrome::Palindrome(std::initializer_list<std::string> words) {
    for (auto w : words) _words.insert(w);
}

bool Palindrome::isPalindrome() const {
    /* for (int i = 0; i < this->length() / 2; ++i) { // For each character in the palindrome (up to half of the palindrome's length)
         // 'std::tolower': 'A' -> 'a'
         if (std::tolower(this->operator[](i)) != std::tolower(this->operator[](this->length() - 1 - i)))
             return false;
     }*/
    return true;
}

const char* Palindrome::toString() const {
    return ""; // this->data();
}
