#ifndef  _Palindrome
#define  _Palindrome

#include <array> // 'std::array' (C++11)
#include <string>
#include <unordered_set> // 'std::unordered_set' (C++11)
#include <utility> // 'std::pair'

class Substituable_character {
private:
    std::unordered_set<char> _substitutes;
public: // 'inline' is for pedagogical reasons *ONLY* (compactness of code)
    const char character;

    inline Substituable_character(char character, std::initializer_list<char> substitutes) : character(character) {
        for (auto substitute : substitutes) _substitutes.insert(substitute);
    }
    // Functor used by 'std::transform' ('()' operator can take as many parameters as desired):

    inline const char operator()(char substitute) const { // '()' operator contextual definition -> functor (https://www.geeksforgeeks.org/functors-in-cpp/)
        if (_substitutes.find(substitute) == _substitutes.end()) return substitute;
        return character;
    }
    std::string toString() const;
};

class Palindrome_word {
private:
    std::string _word;
public:

    inline Palindrome_word(const std::string& word) : _word(word) {
    } // 'inline' is for pedagogical reasons *ONLY* (compactness of code)
    void alter(const Substituable_character&);
    void alter_(const Substituable_character&);

    inline const char* toString() const {
        return _word.data();
    }; // 'inline' is for pedagogical reasons *ONLY* (compactness of code)
};

class Palindrome {
private:
    // 'std::array' is an aggregate type:
    static const std::array<std::pair<char, char>, 2> _Intervals;
    std::unordered_set<std::string> _words;
public:
    Palindrome(std::initializer_list<std::string>);
    virtual bool isPalindrome() const;
    const char* toString() const;
};

#endif
