#ifndef  _Utilities
#define  _Utilities

#include <array> // 'std::array' (C++11)
#include <numeric> // 'std::accumulate'

template<typename Cumulative, typename... Cumulatives> Cumulative Cumulate(Cumulative const& init, Cumulatives const&... cumulatives) {
	std::array<Cumulative, sizeof...(cumulatives) + 1 > my_array{ init, cumulatives... }; // C++20: 'auto my_array = std::to_array(init,cumulatives...);'
	// C++17 fold:
	return (cumulatives + ... + init);
	// Simple example of STL iterators' usage:
	// return std::accumulate(std::next(my_array.begin()), my_array.end(), *my_array.begin());
}

template<typename... Cumulatives> auto Cumulate_to_illustrate_lambda_expression(Cumulatives... cumulatives) {
	return[cumulatives...]() noexcept {
		return ::Cumulate(cumulatives...);
	}(); // Arrrggglll, immediate call...
}

/* Transform 'std::array' into pack... */
// https://stackoverflow.com/questions/35232057/fixed-size-container-to-variadic-template-argument-list-conversion

// C++14, 'std::index_sequence<I...>

template<typename T, std::size_t N, std::size_t... I> auto _std__array_to_pack(const std::array<T, N>& my_array, std::index_sequence<I...>) {
	return ::Cumulate(my_array[I]...);
}

// C++14, 'std::make_index_sequence<N>' -> 0, 1... N-1

template<typename T, std::size_t N, typename Indices = std::make_index_sequence<N> > auto std__array_to_pack(const std::array<T, N>& my_array) {
	return _std__array_to_pack(my_array, Indices());
}

#endif
